@@ -72,6 +72,7 @@ struct _cairo_bo_edge {
     cairo_edge_t edge;
     cairo_bo_edge_t *prev;
     cairo_bo_edge_t *next;
+    cairo_bo_edge_t *colinear;
     cairo_bo_trap_t deferred_trap;
 };
 
@@ -561,8 +562,8 @@ _line_equal (const cairo_line_t *a, const cairo_line_t *b)
            a->p2.x == b->p2.x && a->p2.y == b->p2.y;
 }
 
-static int
-_cairo_bo_sweep_line_compare_edges (cairo_bo_sweep_line_t	*sweep_line,
+static inline int
+_cairo_bo_sweep_line_compare_edges (const cairo_bo_sweep_line_t	*sweep_line,
 				    const cairo_bo_edge_t	*a,
 				    const cairo_bo_edge_t	*b)
 {
@@ -1118,7 +1119,7 @@ _cairo_bo_sweep_line_init (cairo_bo_sweep_line_t *sweep_line)
     sweep_line->current_edge = NULL;
 }
 
-static cairo_status_t
+static void
 _cairo_bo_sweep_line_insert (cairo_bo_sweep_line_t	*sweep_line,
 			     cairo_bo_edge_t		*edge)
 {
@@ -1175,8 +1176,6 @@ _cairo_bo_sweep_line_insert (cairo_bo_sweep_line_t	*sweep_line,
     }
 
     sweep_line->current_edge = edge;
-
-    return CAIRO_STATUS_SUCCESS;
 }
 
 static void
@@ -1310,41 +1309,61 @@ event_log (const char *fmt, ...)
 }
 #endif
 
+#define HAS_COLINEAR(a, b) ((cairo_bo_edge_t *)(((uintptr_t)(a))&~1) == (b))
+#define IS_COLINEAR(e) (((uintptr_t)(e))&1)
+#define MARK_COLINEAR(e, v) ((cairo_bo_edge_t *)(((uintptr_t)(e))|(v)))
+
 static inline cairo_bool_t
-edges_colinear (const cairo_bo_edge_t *a, const cairo_bo_edge_t *b)
+edges_colinear (cairo_bo_edge_t *a, const cairo_bo_edge_t *b)
 {
     unsigned p;
 
+    if (HAS_COLINEAR(a->colinear, b))
+	return IS_COLINEAR(a->colinear);
+
+    if (HAS_COLINEAR(b->colinear, a)) {
+	p = IS_COLINEAR(b->colinear);
+	a->colinear = MARK_COLINEAR(b, p);
+	return p;
+    }
+
     p = 0;
     p |= (a->edge.line.p1.x == b->edge.line.p1.x) << 0;
     p |= (a->edge.line.p1.y == b->edge.line.p1.y) << 1;
     p |= (a->edge.line.p2.x == b->edge.line.p2.x) << 3;
     p |= (a->edge.line.p2.y == b->edge.line.p2.y) << 4;
-    if (p == ((1 << 0) | (1 << 1) | (1 << 3) | (1 << 4)))
+    if (p == ((1 << 0) | (1 << 1) | (1 << 3) | (1 << 4))) {
+	a->colinear = MARK_COLINEAR(b, 1);
 	return TRUE;
+    }
 
-    if (_slope_compare (a, b))
+    if (_slope_compare (a, b)) {
+	a->colinear = MARK_COLINEAR(b, 0);
 	return FALSE;
+    }
 
     /* The choice of y is not truly arbitrary since we must guarantee that it
      * is greater than the start of either line.
      */
     if (p != 0) {
 	/* colinear if either end-point are coincident */
-	return ((p >> 1) & p) != 0;
+	p = (((p >> 1) & p) & 5) != 0;
     } else if (a->edge.line.p1.y < b->edge.line.p1.y) {
-	return edge_compare_for_y_against_x (b,
-					     a->edge.line.p1.y,
-					     a->edge.line.p1.x) == 0;
+	p = edge_compare_for_y_against_x (b,
+					  a->edge.line.p1.y,
+					  a->edge.line.p1.x) == 0;
     } else {
-	return edge_compare_for_y_against_x (a,
-					     b->edge.line.p1.y,
-					     b->edge.line.p1.x) == 0;
+	p = edge_compare_for_y_against_x (a,
+					  b->edge.line.p1.y,
+					  b->edge.line.p1.x) == 0;
     }
+
+    a->colinear = MARK_COLINEAR(b, p);
+    return p;
 }
 
 /* Adds the trapezoid, if any, of the left edge to the #cairo_traps_t */
-static cairo_status_t
+static void
 _cairo_bo_edge_end_trap (cairo_bo_edge_t	*left,
 			 int32_t		 bot,
 			 cairo_traps_t	        *traps)
@@ -1376,8 +1395,6 @@ _cairo_bo_edge_end_trap (cairo_bo_edge_t	*left,
     }
 
     trap->right = NULL;
-
-    return _cairo_traps_status (traps);
 }
 
 
@@ -1386,16 +1403,14 @@ _cairo_bo_edge_end_trap (cairo_bo_edge_t	*left,
  * then either add it to the traps in `traps', if the trapezoid's
  * right edge differs from `edge->next', or do nothing if the new
  * trapezoid would be a continuation of the existing one. */
-static inline cairo_status_t
+static inline void
 _cairo_bo_edge_start_or_continue_trap (cairo_bo_edge_t	*left,
 				       cairo_bo_edge_t  *right,
 				       int               top,
 				       cairo_traps_t	*traps)
 {
-    cairo_status_t status;
-
     if (left->deferred_trap.right == right)
-	return CAIRO_STATUS_SUCCESS;
+	return;
 
     assert (right);
     if (left->deferred_trap.right != NULL) {
@@ -1403,12 +1418,10 @@ _cairo_bo_edge_start_or_continue_trap (cairo_bo_edge_t	*left,
 	{
 	    /* continuation on right, so just swap edges */
 	    left->deferred_trap.right = right;
-	    return CAIRO_STATUS_SUCCESS;
+	    return;
 	}
 
-	status = _cairo_bo_edge_end_trap (left, top, traps);
-	if (unlikely (status))
-	    return status;
+	_cairo_bo_edge_end_trap (left, top, traps);
     }
 
     if (! edges_colinear (left, right)) {
@@ -1422,18 +1435,15 @@ _cairo_bo_edge_start_or_continue_trap (cairo_bo_edge_t	*left,
 		   top);
 #endif
     }
-
-    return CAIRO_STATUS_SUCCESS;
 }
 
-static inline cairo_status_t
+static inline void
 _active_edges_to_traps (cairo_bo_edge_t	*pos,
 			int32_t		 top,
 			unsigned	 mask,
 			cairo_traps_t        *traps)
 {
     cairo_bo_edge_t *left;
-    cairo_status_t status;
     int in_out;
 
 
@@ -1457,9 +1467,7 @@ _active_edges_to_traps (cairo_bo_edge_t	*pos,
 	    }
 	    else
 	    {
-		status = _cairo_bo_edge_end_trap (pos, top, traps);
-		if (unlikely (status))
-		    return status;
+		_cairo_bo_edge_end_trap (pos, top, traps);
 	    }
 	}
 
@@ -1467,22 +1475,15 @@ _active_edges_to_traps (cairo_bo_edge_t	*pos,
 	if ((in_out & mask) == 0) {
 	    /* skip co-linear edges */
 	    if (pos->next == NULL || ! edges_colinear (pos, pos->next)) {
-		status = _cairo_bo_edge_start_or_continue_trap (left, pos,
-								top, traps);
-		if (unlikely (status))
-		    return status;
-
+		_cairo_bo_edge_start_or_continue_trap (left, pos, top, traps);
 		left = pos->next;
 	    }
 	}
 
 	pos = pos->next;
     }
-
-    return CAIRO_STATUS_SUCCESS;
 }
 
-
 /* Execute a single pass of the Bentley-Ottmann algorithm on edges,
  * generating trapezoids according to the fill_rule and appending them
  * to traps. */
@@ -1493,7 +1494,7 @@ _cairo_bentley_ottmann_tessellate_bo_edges (cairo_bo_event_t   **start_events,
 					    cairo_traps_t	*traps,
 					    int			*num_intersections)
 {
-    cairo_status_t status = CAIRO_STATUS_SUCCESS; /* silence compiler */
+    cairo_status_t status;
     int intersection_count = 0;
     cairo_bo_event_queue_t event_queue;
     cairo_bo_sweep_line_t sweep_line;
@@ -1534,20 +1535,16 @@ _cairo_bentley_ottmann_tessellate_bo_edges (cairo_bo_event_t   **start_events,
 	if (event->point.y != sweep_line.current_y) {
 	    for (e1 = sweep_line.stopped; e1; e1 = e1->next) {
 		if (e1->deferred_trap.right != NULL) {
-		    status = _cairo_bo_edge_end_trap (e1,
-						      e1->edge.bottom,
-						      traps);
-		    if (unlikely (status))
-			goto unwind;
+		    _cairo_bo_edge_end_trap (e1,
+					     e1->edge.bottom,
+					     traps);
 		}
 	    }
 	    sweep_line.stopped = NULL;
 
-	    status = _active_edges_to_traps (sweep_line.head,
-					     sweep_line.current_y,
-					     fill_rule, traps);
-	    if (unlikely (status))
-		goto unwind;
+	    _active_edges_to_traps (sweep_line.head,
+				    sweep_line.current_y,
+				    fill_rule, traps);
 
 	    sweep_line.current_y = event->point.y;
 	}
@@ -1565,9 +1562,7 @@ _cairo_bentley_ottmann_tessellate_bo_edges (cairo_bo_event_t   **start_events,
 	case CAIRO_BO_EVENT_TYPE_START:
 	    e1 = &((cairo_bo_start_event_t *) event)->edge;
 
-	    status = _cairo_bo_sweep_line_insert (&sweep_line, e1);
-	    if (unlikely (status))
-		goto unwind;
+	    _cairo_bo_sweep_line_insert (&sweep_line, e1);
 
 	    status = _cairo_bo_event_queue_insert_stop (&event_queue, e1);
 	    if (unlikely (status))
@@ -1670,11 +1665,10 @@ _cairo_bentley_ottmann_tessellate_bo_edges (cairo_bo_event_t   **start_events,
     *num_intersections = intersection_count;
     for (e1 = sweep_line.stopped; e1; e1 = e1->next) {
 	if (e1->deferred_trap.right != NULL) {
-	    status = _cairo_bo_edge_end_trap (e1, e1->edge.bottom, traps);
-	    if (unlikely (status))
-		break;
+	    _cairo_bo_edge_end_trap (e1, e1->edge.bottom, traps);
 	}
     }
+    status = traps->status;
  unwind:
     _cairo_bo_event_queue_fini (&event_queue);
 
@@ -1691,7 +1685,6 @@ _cairo_bentley_ottmann_tessellate_polygon (cairo_traps_t	 *traps,
 					   cairo_fill_rule_t	  fill_rule)
 {
     int intersections;
-    cairo_status_t status;
     cairo_bo_start_event_t stack_events[CAIRO_STACK_ARRAY_LENGTH (cairo_bo_start_event_t)];
     cairo_bo_start_event_t *events;
     cairo_bo_event_t *stack_event_ptrs[ARRAY_LENGTH (stack_events) + 1];
@@ -1699,6 +1692,7 @@ _cairo_bentley_ottmann_tessellate_polygon (cairo_traps_t	 *traps,
     cairo_bo_start_event_t *stack_event_y[64];
     cairo_bo_start_event_t **event_y = NULL;
     int i, num_events, y, ymin, ymax;
+    cairo_status_t status;
 
     num_events = polygon->num_edges;
     if (unlikely (0 == num_events))
@@ -1739,6 +1733,7 @@ _cairo_bentley_ottmann_tessellate_polygon (cairo_traps_t	 *traps,
 	events[i].edge.deferred_trap.right = NULL;
 	events[i].edge.prev = NULL;
 	events[i].edge.next = NULL;
+	events[i].edge.colinear = NULL;
 
 	if (event_y) {
 	    y = _cairo_fixed_integer_floor (events[i].point.y) - ymin;
