@@ -813,6 +813,13 @@ _do_cairo_gstate_user_to_backend (cairo_gstate_t *gstate, double *x, double *y)
     cairo_matrix_transform_point (&gstate->target->device_transform, x, y);
 }
 
+void
+_do_cairo_gstate_user_to_backend_distance (cairo_gstate_t *gstate, double *x, double *y)
+{
+    cairo_matrix_transform_distance (&gstate->ctm, x, y);
+    cairo_matrix_transform_distance (&gstate->target->device_transform, x, y);
+}
+
 void
 _do_cairo_gstate_backend_to_user (cairo_gstate_t *gstate, double *x, double *y)
 {
@@ -820,6 +827,13 @@ _do_cairo_gstate_backend_to_user (cairo_gstate_t *gstate, double *x, double *y)
     cairo_matrix_transform_point (&gstate->ctm_inverse, x, y);
 }
 
+void
+_do_cairo_gstate_backend_to_user_distance (cairo_gstate_t *gstate, double *x, double *y)
+{
+    cairo_matrix_transform_distance (&gstate->target->device_transform_inverse, x, y);
+    cairo_matrix_transform_distance (&gstate->ctm_inverse, x, y);
+}
+
 void
 _cairo_gstate_backend_to_user_rectangle (cairo_gstate_t *gstate,
                                          double *x1, double *y1,
@@ -1221,12 +1235,12 @@ _cairo_gstate_in_stroke (cairo_gstate_t	    *gstate,
     _cairo_traps_init (&traps);
     _cairo_traps_limit (&traps, &limit, 1);
 
-    status = _cairo_path_fixed_stroke_to_traps (path,
-						&gstate->stroke_style,
-						&gstate->ctm,
-						&gstate->ctm_inverse,
-						gstate->tolerance,
-						&traps);
+    status = _cairo_path_fixed_stroke_polygon_to_traps (path,
+							&gstate->stroke_style,
+							&gstate->ctm,
+							&gstate->ctm_inverse,
+							gstate->tolerance,
+							&traps);
     if (unlikely (status))
 	goto BAIL;
 
@@ -1451,12 +1465,12 @@ _cairo_gstate_stroke_extents (cairo_gstate_t	 *gstate,
 	cairo_traps_t traps;
 
 	_cairo_traps_init (&traps);
-	status = _cairo_path_fixed_stroke_to_traps (path,
-						    &gstate->stroke_style,
-						    &gstate->ctm,
-						    &gstate->ctm_inverse,
-						    gstate->tolerance,
-						    &traps);
+	status = _cairo_path_fixed_stroke_polygon_to_traps (path,
+							    &gstate->stroke_style,
+							    &gstate->ctm,
+							    &gstate->ctm_inverse,
+							    gstate->tolerance,
+							    &traps);
 	empty = traps.num_traps == 0;
 	if (! empty)
 	    _cairo_traps_extents (&traps, &extents);
