@@ -85,22 +85,10 @@ _cairo_boxes_get_extents (const cairo_box_t *boxes,
 			  int num_boxes,
 			  cairo_box_t *extents)
 {
-    int n;
-
     assert (num_boxes > 0);
     *extents = *boxes;
-
-    for (n = 1; n < num_boxes; n++) {
-	if (boxes[n].p1.x < extents->p1.x)
-	    extents->p1.x = boxes[n].p1.x;
-	if (boxes[n].p2.x > extents->p2.x)
-	    extents->p2.x = boxes[n].p2.x;
-
-	if (boxes[n].p1.y < extents->p1.y)
-	    extents->p1.y = boxes[n].p1.y;
-	if (boxes[n].p2.y > extents->p2.y)
-	    extents->p2.y = boxes[n].p2.y;
-    }
+    while (--num_boxes)
+	_cairo_box_add_box (extents, ++boxes);
 }
 
 /* XXX We currently have a confusing mix of boxes and rectangles as
@@ -201,7 +189,7 @@ _cairo_rectangle_union (cairo_rectangle_int_t *dst,
  */
 
 cairo_bool_t
-_cairo_box_intersects_line_segment (cairo_box_t *box, cairo_line_t *line)
+_cairo_box_intersects_line_segment (const cairo_box_t *box, cairo_line_t *line)
 {
     cairo_fixed_t t1=0, t2=0, t3=0, t4=0;
     cairo_int64_t t1y, t2y, t3x, t4x;
