@@ -80,6 +80,7 @@
 #if CAIRO_HAS_QUARTZ_SURFACE
 #define Cursor QuartzCursor
 #include <cairo-quartz.h>
+#undef Cursor
 #endif
 #if CAIRO_HAS_SVG_SURFACE
 #include <cairo-svg.h>
@@ -93,9 +94,9 @@
 #if CAIRO_HAS_XLIB_SURFACE
 #define Cursor XCursor
 #include <cairo-xlib.h>
+#undef Cursor
 #endif
 
-#define ARRAY_LENGTH(array) (sizeof (array) / sizeof ((array)[0]))
 #define surface_has_type(surface,type) (cairo_surface_get_type (surface) == (type))
 
 typedef cairo_test_status_t (* surface_test_func_t) (cairo_surface_t *surface);
@@ -560,6 +561,13 @@ test_cairo_xcb_surface_set_size (cairo_surface_t *surface)
     return CAIRO_TEST_SUCCESS;
 }
 
+static cairo_test_status_t
+test_cairo_xcb_surface_set_drawable (cairo_surface_t *surface)
+{
+    cairo_xcb_surface_set_drawable (surface, 0, 5, 5);
+    return CAIRO_TEST_SUCCESS;
+}
+
 #endif
 
 #if CAIRO_HAS_XLIB_SURFACE
@@ -703,6 +711,7 @@ struct {
 #endif
 #if CAIRO_HAS_XCB_SURFACE
     TEST (cairo_xcb_surface_set_size, CAIRO_SURFACE_TYPE_XCB, TRUE),
+    TEST (cairo_xcb_surface_set_drawable, CAIRO_SURFACE_TYPE_XCB, TRUE),
 #endif
 #if CAIRO_HAS_XLIB_SURFACE
     TEST (cairo_xlib_surface_set_size, CAIRO_SURFACE_TYPE_XLIB, TRUE),
